@@ -49,11 +49,15 @@ from the X Consortium.
 
 static const char *kumppa_defaults [] ={
   ".background:		black",
+  "*fpsSolid:		true",
   "*speed:		0.1",
   "*delay:		10000",
   "*random:		True",
   /* leave this off by default, since it slows things down.  -- jwz. */
   "*useDBE:		False",
+#ifdef USE_IPHONE
+  "*ignoreRotation:     True",
+#endif
   0
 };
 
